@@ -447,7 +447,7 @@ class GantryAlerts:
         notification_name: str,
         slack_webhook_url: str,
         notify_daily: bool,
-        daily_notification_time: str = None,
+        daily_notification_time: Optional[str] = None,
     ) -> str:
         """
         An alert notification sets up the means by which you will be notified
