@@ -15,6 +15,7 @@ from urllib.parse import urlparse, urlunparse
 import click
 import requests
 import yaml
+from click_aliases import ClickAliasedGroup
 
 import gantry
 from gantry.api_client import APIClient
@@ -36,7 +37,7 @@ PENDING_TASK_STATUSES = [
 ]
 
 
-@click.group()
+@click.group(cls=ClickAliasedGroup)
 def projection():
     """
     Use this to register a new custom projection to Gantry.
@@ -394,7 +395,7 @@ def update_lambda_dev(projection_dir, dry_run):
     click.secho(f"SUCCESS. Task {task_id} has been submitted.", fg="cyan")
 
 
-@projection.command("update")
+@projection.command(aliases=["create"], help="Create or update a custom projection in Gantry")
 @click.option("--projection-dir", type=click.Path())
 @click.option(
     "--dry-run",
@@ -460,7 +461,7 @@ def update(projection_dir, dry_run):
     _call_get_logs(api_client, task_id)
 
 
-@projection.command("get-logs")
+@projection.command("get-logs", help="Display custom projection build logs given task ID")
 @click.option("--task-id", type=click.STRING)
 def get_logs(task_id):
     gantry.init()
