@@ -1,9 +1,12 @@
+import logging
 import uuid
 from typing import Dict, List, Optional
 
 from gantry.api_client import APIClient
 from gantry.dataset.gantry_dataset import GantryDataset
 
+logger = logging.getLogger(__name__)
+
 
 class GantryDatasetClient:
     def __init__(
@@ -77,6 +80,9 @@ class GantryDatasetClient:
         """
         res = self._api_client.request("GET", f"/api/v1/datasets/{name}", raise_for_status=True)
 
+        if res["data"]["disabled"]:
+            logger.warning("This dataset is marked as deleted")
+
         return GantryDataset(
             api_client=self._api_client,
             user_email=self.email,
@@ -98,20 +104,65 @@ class GantryDatasetClient:
         Returns:
             list of commit info
         """
-        dataset_id = self.get_dataset(name)._dataset_id
-        res = self._api_client.request(
-            "GET", f"/api/v1/datasets/{dataset_id}/commits", raise_for_status=True
-        )
+        dataset = self.get_dataset(name)
 
-        return res["data"]
+        return dataset.list_commits()
 
-    def list_datasets(self) -> List[Dict[str, str]]:
+    def list_datasets(self, include_deleted=False) -> List[Dict[str, str]]:
         """
         List datasets
 
+        Args:
+            include_deleted (bool): whether to return deleted datasets, defaults to false
+
         Returns:
             List of dataset information
         """
-        res = self._api_client.request("GET", "/api/v1/datasets", raise_for_status=True)
+        res = self._api_client.request(
+            "GET",
+            "/api/v1/datasets",
+            raise_for_status=True,
+            params={"include_deleted": include_deleted},
+        )
 
         return res["data"]
+
+    def delete_dataset(self, name: str):
+        """
+        Delete dataset
+
+        Args:
+            name (str): dataset name
+
+        Returns:
+            none
+        """
+        dataset = self.get_dataset(name)
+
+        return dataset.delete()
+
+    def ping(self) -> bool:
+        """
+        Pings the API client server to check if it is alive.
+        Returns True if alive, False if there is an error during ping process.
+        """
+        try:
+            # Cannot use /healthz/* endpoints as those will be answered by nginx
+            # need to use /.
+            # See https://linear.app/gantry/issue/ENG-2978/revisit-ping-in-sdk
+            self._api_client.request("GET", "/api/ping", raise_for_status=True)
+            return True
+        except Exception as e:
+            logger.error(f"Error during ping: {e}")
+            return False
+
+    def ready(self) -> bool:
+        """
+        Checks whether the API is ready to receive traffic with the provided API Key.
+        """
+        try:
+            self._api_client.request("GET", "/api/v1/auth-ping", raise_for_status=True)
+            return True
+        except Exception as e:
+            logger.error(f"Error during api key check: {e}")
+            return False
