@@ -1,3 +1,6 @@
+import functools
+import hashlib
+import io
 import json
 import logging
 import os
@@ -6,11 +9,13 @@ import uuid
 from collections import defaultdict
 from dataclasses import asdict, dataclass
 from pathlib import Path
-from typing import Dict, List, Optional
+from typing import IO, Callable, Dict, List, Optional
+
+import pandas as pd
 
 from gantry.api_client import APIClient
 from gantry.dataset import constants
-from gantry.exceptions import GantryRequestException
+from gantry.exceptions import DatasetDeletedException, GantryRequestException
 from gantry.utils import (
     download_file_from_url,
     get_files_checksum,
@@ -23,6 +28,29 @@ logger = logging.getLogger(__name__)
 BATCH_SIZE = 5
 
 
+def _ensure_not_deleted(func: Callable) -> Callable:
+    """
+    Decorator to ensure we do not perform an operation on a deleted dataset, and instead warn the
+    user that the dataset is deleted.
+    """
+
+    @functools.wraps(func)
+    def wrapper(self, *args, **kwargs):
+        resp = self._api_client.request(
+            "GET",
+            f"/api/v1/datasets/{self.dataset_name}",
+            raise_for_status=True,
+        )
+        dataset_info = resp["data"]
+
+        if dataset_info["disabled"]:
+            raise DatasetDeletedException("This dataset has been deleted!")
+
+        return func(self, *args, **kwargs)
+
+    return wrapper
+
+
 @dataclass
 class DatasetFileInfo:
     file_name: str
@@ -85,11 +113,12 @@ class GantryDataset:
         """
         diff = self._get_diff()
         return {
-            constants.NEW_FILES: [f.file_name for f in diff[constants.NEW_FILES]],
-            constants.MODIFIED_FILES: [f.file_name for f in diff[constants.MODIFIED_FILES]],
-            constants.DELETED_FILES: [f.file_name for f in diff[constants.DELETED_FILES]],
+            constants.NEW_FILES: [f for f, _ in diff[constants.NEW_FILES]],
+            constants.MODIFIED_FILES: [f for f, _ in diff[constants.MODIFIED_FILES]],
+            constants.DELETED_FILES: [f for f, _ in diff[constants.DELETED_FILES]],
         }
 
+    @_ensure_not_deleted
     def create_commit(self, message: str) -> Dict[str, str]:
         """
         This method will compute the difference between the current commit and the local directory,
@@ -102,7 +131,8 @@ class GantryDataset:
         Returns:
             Commit info
         """
-        diff = self._get_diff()
+        commit_id = uuid.uuid4()
+        diff = self._get_diff_for_commit(commit_id)
 
         if (
             not diff[constants.NEW_FILES]
@@ -134,6 +164,7 @@ class GantryDataset:
                     "message": message,
                     "metadata_s3_file_version": version_id,
                     "parent_commit_id": self._get_current_commit()["id"],
+                    "commit_id": commit_id,
                     "email": self.user_email,
                 },
                 raise_for_status=True,
@@ -150,6 +181,135 @@ class GantryDataset:
 
         return commit_info
 
+    @_ensure_not_deleted
+    def add_file(
+        self,
+        file: IO,
+        filename: Optional[str] = None,
+        commit_message: Optional[str] = None,
+    ) -> Dict[str, str]:
+        """
+        This method will upload a file (such as a file opened with ``open()``) to the dataset in a
+        new commit. Does not rely on the user having the dataset synced, nor does it cause the
+        dataset to sync after the upload.
+
+        Args:
+            file (IO): the file to be uploaded
+            filename (str, Optional): the name of the file to be uploaded. Defaults to the local
+                filename of the file
+            commit_message (str, Optional): the commit message that will be associated with the
+                upload of the file. Defaults to a generic message if not set
+        Returns:
+            Commit info
+        """
+        # TODO: require specification of parent commit
+        if filename:
+            name = filename
+        else:
+            name = os.path.basename(file.name)
+
+        return self._add_single_file(file, name, commit_message)
+
+    @_ensure_not_deleted
+    def add_tabular_file(
+        self,
+        file: IO,
+        filename: Optional[str] = None,
+        commit_message: Optional[str] = None,
+    ) -> Dict[str, str]:
+        """
+        This method will upload a csv file (such as a csv file opened with ``open()``) to the
+        tabular_manifests of the dataset in a new commit. Does not rely on the user having the
+        dataset synced, nor does it cause the dataset to sync after the upload.
+
+        Args:
+            file (IO): the csv file to be uploaded
+            filename (str, Optional): the name of the file to be uploaded. Defaults to the local
+                filename of the file. Name must end with '.csv'
+            commit_message (str, Optional): the commit message that will be associated with the
+                upload of the file. Defaults to a generic message if not set
+        Returns:
+            Commit info
+        """
+        # TODO: require specification of parent commit
+        if filename:
+            name = filename
+        else:
+            name = os.path.basename(file.name)
+
+        if name[-4:] != ".csv":
+            raise ValueError(f"filename must end with '.csv'! Got filename: {name}")
+
+        return self._add_single_file(file, name, commit_message)
+
+    @_ensure_not_deleted
+    def add_dataframe(
+        self,
+        dataframe: pd.DataFrame,
+        csvname: Optional[str] = None,
+        commit_message: Optional[str] = None,
+    ) -> Dict[str, str]:
+        """
+        This method will upload a pandas DataFrame as a new commit in the form of a csv. Does not
+        rely on the user having the dataset synced, nor does it cause the dataset to sync after the
+        upload.
+
+        Args:
+            dataframe (pd.DataFrame): the dataframe to be uploaded
+            csvname (str, Optional): the name for the csv that will be uploaded. Defaults to
+                dataframe-[md5 hash].csv
+            commit_message (str, Optional): the commit message that will be associated with the
+                upload of the file. Defaults to a generic message if not set
+        Returns:
+            Commit info
+        """
+        # TODO: require specification of parent commit
+        if csvname:
+            if csvname[-4:] == ".csv":
+                name = csvname
+            else:
+                name = f"{csvname}.csv"
+        else:
+            hash = hashlib.md5(pd.util.hash_pandas_object(dataframe).values).hexdigest()
+            name = f"dataframe-{hash}.csv"
+
+        file = io.StringIO()
+        dataframe.to_csv(file)
+        file.seek(0)
+
+        return self._add_single_file(file, name, commit_message)
+
+    @_ensure_not_deleted
+    def rollback(self, commit_id: str) -> Dict[str, str]:
+        """
+        Roll back the dataset to a previous version based on commit id, then sync the local dataset
+        folder to the newly created commit.
+
+        Args:
+            commit_id (str): target commit id
+        Returns:
+            commit info
+        """
+        target_commit = self.get_commit(commit_id)
+
+        resp = self._api_client.request(
+            "POST",
+            f"/api/v1/datasets/{self._dataset_id}/commits",
+            json={
+                "message": f"Rollback dataset to commit: {target_commit['id']}",
+                "metadata_s3_file_version": target_commit["metadata_s3_file_version"],
+                "parent_commit_id": self._get_current_commit()["id"],
+                "email": self.user_email,
+            },
+            raise_for_status=True,
+        )
+
+        commit_info = resp["data"]
+        self.sync(commit_info["id"])
+
+        return commit_info
+
+    @_ensure_not_deleted
     def sync(self, commit_id: Optional[str] = None) -> Dict[str, str]:
         """
         Sync local dataset folder based on a commit id. If commit_id was not provided, will sync
@@ -176,7 +336,7 @@ class GantryDataset:
             ]
         )
 
-        diff = self._get_diff(return_local_info=False)
+        diff = self._get_diff_for_sync()
         self._download_files(diff[constants.MODIFIED_FILES])  # Overwrite modified files
         self._download_files(diff[constants.DELETED_FILES])  # redownload deleted files
 
@@ -188,6 +348,18 @@ class GantryDataset:
 
         return target_commit
 
+    def delete(self):
+        """
+        Deletes the dataset (currently it is just marked as deleted)
+        """
+        self._api_client.request(
+            "DELETE",
+            f"/api/v1/datasets/{self._dataset_id}",
+            raise_for_status=True,
+        )
+        logger.info(f"Dataset {self.dataset_name} has been deleted")
+        return
+
     def get_commit(self, commit_id: str) -> Dict[str, str]:
         """
         Get commit details
@@ -215,9 +387,9 @@ class GantryDataset:
         return resp["data"][0]  # return latest commit
 
     def _update_manifest_file(self, diff: Dict[str, List[DatasetFileInfo]]):
-        new_files = dict([(f.file_name, f) for f in diff[constants.NEW_FILES]])
-        modified_files = dict([(f.file_name, f) for f in diff[constants.MODIFIED_FILES]])
-        deleted_files = dict([(f.file_name, f) for f in diff[constants.DELETED_FILES]])
+        new_files = {f.file_name: f for f in diff[constants.NEW_FILES]}
+        modified_files = {f.file_name: f for f in diff[constants.MODIFIED_FILES]}
+        deleted_files = {f.file_name: f for f in diff[constants.DELETED_FILES]}
 
         with open(
             self._get_file_path(f"{constants.DATASET_MANIFEST_FILE}{constants.NEW_SUFFIX}"), "w"
@@ -244,19 +416,95 @@ class GantryDataset:
             self._get_file_path(constants.DATASET_MANIFEST_FILE),
         )
 
-    def _get_diff(self, return_local_info=True):
+    def _get_diff_for_commit(self, commit_id: uuid.UUID):
+        repo_diff = self._get_diff()
+        commit_diff = {}
+
+        # generate new url for new files
+        commit_diff[constants.NEW_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url=self._get_s3_url(file_name, commit_id),
+                sha256=checksum,
+            )
+            for file_name, checksum in repo_diff[constants.NEW_FILES]
+        ]
+        # generate new url for modified files
+        commit_diff[constants.MODIFIED_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url=self._get_s3_url(file_name, commit_id),
+                sha256=checksum,
+            )
+            for file_name, checksum in repo_diff[constants.MODIFIED_FILES]
+        ]
+        # don't need url for deleted files
+        commit_diff[constants.DELETED_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url="",
+                sha256=checksum,
+            )
+            for file_name, checksum in repo_diff[constants.DELETED_FILES]
+        ]
+
+        return commit_diff
+
+    def _get_diff_for_sync(self):
+        repo_diff = self._get_diff()
+        sync_diff = {}
+
+        commit_snapshot = {}
+        with open(self._get_file_path(constants.DATASET_MANIFEST_FILE)) as cur_gantry_manifest:
+            for line in cur_gantry_manifest:
+                file_info = json.loads(line)
+                commit_snapshot[file_info[constants.FILE_NAME]] = file_info
+
+        # don't need url for new files
+        sync_diff[constants.NEW_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url="",
+                sha256=checksum,
+            )
+            for file_name, checksum in repo_diff[constants.NEW_FILES]
+        ]
+        # retrieve url for modified files
+        sync_diff[constants.MODIFIED_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url=commit_snapshot[file_name][constants.URL],
+                sha256=commit_snapshot[file_name][constants.SHA256],
+                version_id=commit_snapshot[file_name][constants.VERSION_ID],
+            )
+            for file_name, _ in repo_diff[constants.MODIFIED_FILES]
+        ]
+        # retrieve url for deleted files
+        sync_diff[constants.DELETED_FILES] = [
+            DatasetFileInfo(
+                file_name=file_name,
+                url=commit_snapshot[file_name][constants.URL],
+                sha256=commit_snapshot[file_name][constants.SHA256],
+                version_id=commit_snapshot[file_name][constants.VERSION_ID],
+            )
+            for file_name, _ in repo_diff[constants.DELETED_FILES]
+        ]
+
+        return sync_diff
+
+    def _get_diff(self):
         """
         Generate local diff.
 
         Args:
-            return_local_info (bool): if true return local file info for modiflied files else
+            return_local_info (bool): if true return local file info for modified files else
             return file info in dataset repo
 
         Returns:
             {
-                "new_files": List[DatasetFileInfo],
-                "modified_files": List[DatasetFileInfo],
-                "deleted_files": List[DatasetFileInfo],
+                "new_files": List[Tuple(str,str)],
+                "modified_files": List[Tuple(str,str)],
+                "deleted_files": List[Tuple(str,str)],
             }
         """
         repo_diff = defaultdict(list)
@@ -274,51 +522,55 @@ class GantryDataset:
             if file_name.startswith(constants.GANTRY_FOLDER):  # skip .dataset_metadata folder
                 continue
             elif file_name not in commit_snapshot:  # new added file
-                repo_diff[constants.NEW_FILES].append(
-                    DatasetFileInfo(
-                        file_name=file_name,
-                        url=self._get_s3_url(file_name),
-                        sha256=checksum,
-                    )
-                )
+                repo_diff[constants.NEW_FILES].append((file_name, checksum))
             elif checksum != commit_snapshot[file_name].get(
                 constants.SHA256
             ):  # if file_name in current_snapshot --> check if it has been modified
-                if return_local_info:
-                    repo_diff[constants.MODIFIED_FILES].append(
-                        DatasetFileInfo(
-                            file_name=file_name,
-                            url=self._get_s3_url(file_name),
-                            sha256=checksum,
-                        )
-                    )
-                else:
-                    repo_diff[constants.MODIFIED_FILES].append(
-                        DatasetFileInfo(
-                            file_name=file_name,
-                            url=commit_snapshot[file_name][constants.URL],
-                            version_id=commit_snapshot[file_name][constants.VERSION_ID],
-                            sha256=commit_snapshot[file_name][constants.SHA256],
-                        )
-                    )
+                repo_diff[constants.MODIFIED_FILES].append((file_name, checksum))
 
         for file_name in commit_snapshot:
             if file_name not in local_files:
-                repo_diff[constants.DELETED_FILES].append(
-                    DatasetFileInfo(
-                        file_name=file_name,
-                        url=commit_snapshot[file_name][constants.URL],
-                        sha256=commit_snapshot[file_name][constants.SHA256],
-                        version_id=commit_snapshot[file_name][constants.VERSION_ID],
-                    )
-                )
+                repo_diff[constants.DELETED_FILES].append((file_name, checksum))
+
         return repo_diff
 
+    def _add_single_file(self, file: IO, filename: str, commit_msg: Optional[str]):
+        if commit_msg is None:
+            commit_msg = f"Uploaded {filename} to dataset"
+
+        data = {
+            "email": self.user_email,
+            "commit_msg": commit_msg,
+        }
+        files = {
+            "file": (filename, file),
+        }
+
+        resp = self._api_client.request(
+            "POST",
+            f"/api/v1/datasets/{self._dataset_id}/file",
+            files=files,
+            data=data,
+            raise_for_status=True,
+        )
+
+        commit_info = resp["data"]
+        return commit_info
+
     def _get_file_path(self, file_name: str) -> str:
         return os.path.join(self.workspace, self.dataset_name, file_name)
 
-    def _get_s3_url(self, file_name: str) -> str:
-        return f"s3://{self._bucket_name}/{self._dataset_s3_prefix}/{self.dataset_name}/{file_name}"
+    def _get_s3_url(self, file_name: str, commit_id: Optional[uuid.UUID] = None) -> str:
+        if commit_id:
+            return (
+                f"s3://{self._bucket_name}/{self._dataset_s3_prefix}/"
+                f"{self.dataset_name}/{commit_id}/{file_name}"
+            )
+        else:
+            return (
+                f"s3://{self._bucket_name}/{self._dataset_s3_prefix}/"
+                f"{self.dataset_name}/{file_name}"
+            )
 
     def _get_obj_key(self, s3_url: str) -> str:
         _, obj_key = parse_s3_path(s3_url)
@@ -397,7 +649,7 @@ class GantryDataset:
 
             for item in batch:
                 local_path = self._get_file_path(item.file_name)
-                presigned_url = presigned_urls.get(self._get_obj_key(item.url))
+                presigned_url = presigned_urls[self._get_obj_key(item.url)]
                 version_id = upload_file_to_url(
                     presigned_url=presigned_url, local_path=Path(local_path)
                 )
