@@ -41,7 +41,9 @@ class QueryError(GantryException):
 
 
 class GantryBatchCreationException(GantryException):
-    """Raised when gantry failed to create batch"""
+    """
+    Raised when gantry failed to create batch
+    """
 
     pass
 
@@ -80,4 +82,13 @@ class GantryRequestException(GantryException):
             msg = "Internal Gantry error. Contact Gantry support for help."
 
         err_msg = "{} (Status code: {})".format(msg, status_code)
+        self.status_code = status_code
         super().__init__(err_msg)
+
+
+class DatasetDeletedException(GantryException):
+    """
+    Raised when an illegal dataset operation is attempted on a deleted dataset
+    """
+
+    pass
