@@ -13,10 +13,9 @@ from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 from warnings import warn
 
+import numpy as np
 import pandas as pd
-import requests
-from tqdm import tqdm
-from typeguard import typechecked
+from typeguard import check_type, typechecked
 
 from gantry.exceptions import GantryBatchCreationException, GantryLoggingException
 from gantry.logger.constants import CHUNK_SIZE, BatchType
@@ -35,6 +34,7 @@ from gantry.logger.utils import (
     _batch_success_msg,
     _build_batch_iterator,
     _check_sample_rate,
+    _concurrent_upload_multipart_batch,
     _is_empty,
     _is_not_empty,
     _log_exception,
@@ -210,17 +210,7 @@ class Gantry:
         upload_id = preupload_res["upload_id"]
         file_key = preupload_res["key"]
 
-        # Upload the parts.
-        parts = []
-        part_num = 1  # AWS S3 needs part num to start with 1.
-        for block in tqdm(
-            data_batch_iterator, desc="Uploading to Gantry Bucket: ", total=total_num_parts
-        ):
-            # TODO: parallelize this client side?
-            response = requests.put(signed_urls[part_num - 1], data=block)
-            etag = str(response.headers["ETag"])
-            parts.append({"ETag": etag, "PartNumber": part_num})
-            part_num += 1
+        parts = _concurrent_upload_multipart_batch(data_batch_iterator, signed_urls)
 
         logger.info("Starting Gantry Ingestion")
         # Submit file completion to gantry API.
@@ -250,13 +240,13 @@ class Gantry:
         self,
         application: str,
         version: Optional[Union[int, str]] = None,
-        inputs: Optional[Union[List[dict], pd.DataFrame, pd.Series]] = None,
-        outputs: Optional[Union[List[Any], List[dict], pd.DataFrame, pd.Series]] = None,
+        inputs: Optional[Union[List[dict], pd.DataFrame, pd.Series, np.ndarray]] = None,
+        outputs: Optional[Union[List[Any], List[dict], pd.DataFrame, pd.Series, np.ndarray]] = None,
         feedback_keys: Optional[List[str]] = None,
         feedback_ids: Optional[Union[List[str], List[dict]]] = None,
-        feedbacks: Optional[Union[List[dict], pd.DataFrame, pd.Series]] = None,
+        feedbacks: Optional[Union[List[dict], pd.DataFrame, pd.Series, np.ndarray]] = None,
         ignore_inputs: Optional[List[str]] = None,
-        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
+        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
@@ -427,13 +417,13 @@ class Gantry:
     def _log_predictions_and_feedbacks(
         self,
         application: str,
-        inputs: Union[List[dict], pd.DataFrame],
-        outputs: Union[List[dict], pd.DataFrame],
-        feedbacks: Union[List[dict], pd.DataFrame],
+        inputs: Union[List[dict], pd.DataFrame, np.ndarray],
+        outputs: Union[List[Any], List[dict], pd.DataFrame, np.ndarray],
+        feedbacks: Union[List[dict], pd.DataFrame, np.ndarray],
         version: Optional[Union[int, str]] = None,
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
-        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
+        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
@@ -467,6 +457,12 @@ class Gantry:
         feedbacks = serializable_value(feedbacks)
         version = serializable_value(version)
 
+        # typecheck everything, relevant when inputs are numpy arrays
+        check_type("inputs", inputs, List[dict])
+        check_type("feedbacks", feedbacks, List[dict])
+        if isinstance(timestamps, np.ndarray):
+            check_type("timestamps", timestamps.tolist(), List[datetime.datetime])
+
         environment, context = self._get_env_and_context(tags)
 
         timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, size)
@@ -741,9 +737,9 @@ class Gantry:
         self,
         application: str,
         feedback_ids: Union[List[str], List[dict]],
-        feedbacks: Union[List[dict], pd.DataFrame],
+        feedbacks: Union[List[dict], pd.DataFrame, np.ndarray],
         feedback_version: Optional[Union[str, int]] = None,
-        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
+        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
@@ -784,6 +780,11 @@ class Gantry:
             feedbacks = serializable_value(feedbacks)
             serialized_version: Union[str, int] = serializable_value(feedback_version)
 
+            # typecheck everything, relevant when inputs are numpy arrays
+            check_type("feedbacks", feedbacks, List[dict])
+            if isinstance(timestamps, np.ndarray):
+                check_type("timestamps", timestamps.tolist(), List[datetime.datetime])
+
             timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, len(feedbacks))
 
             if len(feedback_ids) == len(feedbacks):
@@ -933,13 +934,13 @@ class Gantry:
     def log_predictions(
         self,
         application: str,
-        inputs: Union[List[dict], pd.DataFrame],
-        outputs: Union[List[dict], List[Any], pd.DataFrame],
+        inputs: Union[List[dict], pd.DataFrame, np.ndarray],
+        outputs: Union[List[dict], List[Any], pd.DataFrame, np.ndarray],
         version: Optional[Union[int, str]] = None,
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
         feedback_ids: Optional[Union[List[str], List[dict]]] = None,
-        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
+        timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
@@ -977,6 +978,11 @@ class Gantry:
             outputs = serializable_value(outputs)
             version = serializable_value(version) if version is not None else None
 
+            # typecheck everything, relevant when inputs are numpy arrays
+            check_type("inputs", inputs, List[dict])
+            if isinstance(timestamps, np.ndarray):
+                check_type("timestamps", timestamps.tolist(), List[datetime.datetime])
+
             if len(inputs) == len(outputs):
                 if timestamps is not None and not len(timestamps) == len(inputs):
                     raise GantryLoggingException(
