@@ -4,8 +4,6 @@ import random
 import uuid
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
-import pandas as pd
-
 from gantry.exceptions import GantryLoggingException
 from gantry.logger.constants import BatchType, Delimiter, UploadFileType
 from gantry.logger.types import DataLink, DataLinkElement
@@ -21,9 +19,9 @@ def _build_prediction_and_feedback_events(
     context: Dict[str, str],
     environment: str,
     timestamp_idx: Iterable,
-    inputs: Union[List[dict], pd.DataFrame],
-    outputs: Union[List[dict], List[Any], pd.DataFrame],
-    feedbacks: Union[List[dict], pd.DataFrame],
+    inputs: List[dict],
+    outputs: Union[List[dict], List[Any]],
+    feedbacks: List[dict],
     version: Optional[Union[str, int]] = None,
     feedback_keys: Optional[List[str]] = None,
     ignore_inputs: Optional[List[str]] = None,
@@ -79,7 +77,9 @@ def _build_prediction_and_feedback_events(
 
 
 def _create_timestamp_idx(
-    sort_on_timestamp: bool, timestamps: Optional[Iterable[Any]], record_count: int
+    sort_on_timestamp: bool,
+    timestamps: Optional[Iterable[Any]],
+    record_count: int,
 ) -> Iterable:
     """
     This function is used to take an index of timestamps, possibly None, and a length,
@@ -152,8 +152,8 @@ def _build_feedback_event(
 
 def _build_prediction_events(
     application: str,
-    inputs: Union[List[dict], pd.DataFrame],
-    outputs: Union[List[dict], List[Any], pd.DataFrame],
+    inputs: List[dict],
+    outputs: Union[List[dict], List[Any]],
     timestamp_idx: Iterable,
     context: Dict[str, str],
     environment: str,
