@@ -50,17 +50,15 @@ LOGGING_LEVEL_T = Literal["DEBUG", "INFO", "WARNING", "CRITICAL", "ERROR"]
 @typechecked
 def _resolve_params(
     api_key: Optional[str] = None,
-    logs_location: Optional[str] = None,
     logging_level: Optional[LOGGING_LEVEL_T] = None,
     environment: Optional[str] = None,
-    bypass_firehose: Optional[bool] = None,
     send_in_background: Optional[bool] = None,
 ) -> Tuple[str, str, LOGGING_LEVEL_T, str, bool, bool]:
     api_key = api_key or os.environ.get("GANTRY_API_KEY")
     if api_key is None:
         raise ValueError("logger was not initialized with API Key")
 
-    logs_location = cast(str, logs_location or os.environ.get("GANTRY_LOGS_LOCATION", PROD_API_URL))
+    logs_location = cast(str, os.environ.get("GANTRY_LOGS_LOCATION", PROD_API_URL))
     logging_level = cast(
         LOGGING_LEVEL_T, logging_level or os.environ.get("GANTRY_LOGGING_LEVEL", "INFO")
     )
@@ -73,8 +71,7 @@ def _resolve_params(
             "logger was not initialized with http endpoint. Only HTTP logs locations supported."
         )
 
-    if bypass_firehose is None:
-        bypass_firehose = bool(strtobool(os.environ.get("GANTRY_BYPASS_FIREHOSE", "false")))
+    bypass_firehose = bool(strtobool(os.environ.get("GANTRY_BYPASS_FIREHOSE", "false")))
 
     if send_in_background is None:
         if "GANTRY_SEND_IN_BACKGROUND" in os.environ:
@@ -88,42 +85,25 @@ def _resolve_params(
     return api_key, logs_location, logging_level, environment, bypass_firehose, send_in_background
 
 
-def init(
+def _init(
     api_key: Optional[str] = None,
-    logs_location: Optional[str] = None,
     logging_level: Optional[LOGGING_LEVEL_T] = None,
     environment: Optional[str] = None,
-    bypass_firehose: Optional[bool] = None,
     send_in_background: Optional[bool] = None,
 ):
     """
     Initialize the logger. Initialization should happen before submitting any data to Gantry.
 
-    Example:
-
-    .. code-block:: python
-
-       import gantry
-       gantry.init(api_key="foobar")
-
-
     Args:
         api_key (optional, str): The Gantry API Key. You can also set this parameter by setting
             the env variable GANTRY_API_KEY.
-        logs_location (optional, str): URL where Gantry records will be sent. You can also
-            set this parameter by setting the env variable GANTRY_LOGS_LOCATION. If not specified,
-            it will default to the prod SaaS URL. DO NOT SPECIFY THIS PARAMETER unless indicated.
-        logging_level (str): Set logging level for Gantry system logging. You can also set this
-            parameter by setting the env variable GANTRY_LOGGING_LEVEL.
+        logging_level (optional, str): Set logging level for Gantry system logging.
+            You can also set this parameter by setting the env variable GANTRY_LOGGING_LEVEL.
+            Options are: DEBUG, INFO, WARNING, CRITICAL or ERROR.
             If not specified, it defaults to INFO.
-        environment (str): Set the value for the environment label attached to data instrumented.
-            You can also set this parameter by setting the env variable GANTRY_ENVIRONMENT.
-            If not provided, it defaults to "dev".
-        bypass_firehose (bool, defaults to False): when in development mode, optionally turn
-            this on to send raw events to the worker rather than going through Firehose.
-            You can also set this parameter by setting the env variable GANTRY_BYPASS_FIREHOSE
-            If not provided, it defaults to False.
-            USE THIS PARAMETER ONLY FOR DEVELOPMENT PURPOSES.
+        environment (optional, str): Set the value for the environment label attached
+            to data instrumented. You can also set this parameter by setting the env
+            variable GANTRY_ENVIRONMENT. If not provided, it defaults to "dev".
         send_in_background (optional, bool): set whether Gantry logging methods should
             run synchronously or not. You can also set this value by setting
             the env variable GANTRY_SEND_IN_BACKGROUND.
@@ -137,9 +117,7 @@ def init(
         environment,
         bypass_firehose,
         send_in_background,
-    ) = _resolve_params(
-        api_key, logs_location, logging_level, environment, bypass_firehose, send_in_background
-    )
+    ) = _resolve_params(api_key, logging_level, environment, send_in_background)
 
     global _CLIENT
 
