@@ -1,12 +1,17 @@
+import concurrent
 import functools
 import json
 import logging
 import os
 import subprocess
+import urllib
 from itertools import islice
-from typing import Any, List, Union
+from typing import Any, Iterable, List, Union
 
+
+import numpy as np
 import pandas as pd
+import requests
 
 from gantry.exceptions import GantryBatchCreationException, GantryLoggingException
 from gantry.logger.stores import APILogStore, BaseLogStore
@@ -45,6 +50,8 @@ def _is_empty(data: Union[List[Any], pd.DataFrame, Any] = None) -> bool:
     """
     if isinstance(data, pd.DataFrame):
         return data.empty
+    elif isinstance(data, np.ndarray):
+        return data.size == 0
     return True if not data else False
 
 
@@ -68,7 +75,11 @@ def _batch_success_msg(batch_id, application, log_store: Union[APILogStore, Base
         else:
             host = "[YOUR_GANTRY_DASHBOARD]"
 
-        logger.info("Track your batch at {}/applications/{}/batches".format(host, application))
+        logger.info(
+            "Track your batch at {}/applications/{}/jobs".format(
+                host, urllib.parse.quote(application)
+            )
+        )
         logger.info("Look for batch id: {}".format(batch_id))
 
 
@@ -105,3 +116,24 @@ def _build_batch_iterator(iterable, batch_size):
         if batch:
             lines += "\n"
         yield lines.encode("utf-8")
+
+
+def _put_block(url: str, part_num: int, block: dict) -> dict:
+    response = requests.put(url, data=block)
+    etag = str(response.headers["ETag"])
+    return {"ETag": etag, "PartNumber": part_num}
+
+
+def _concurrent_upload_multipart_batch(
+    data_batch_iterator: Iterable[dict], signed_urls: List[str]
+) -> List[dict]:
+    """Uploads batches to presigned URLs concurrently using threads"""
+    with concurrent.futures.ThreadPoolExecutor(max_workers=10) as executor:
+        logger.info("Starting multipart upload")
+        futures = [
+            executor.submit(_put_block, signed_urls[i], i + 1, block)
+            for i, block in enumerate(data_batch_iterator)
+        ]
+        parts = [f.result() for f in concurrent.futures.as_completed(futures)]
+        logger.info("Multipart upload complete")
+        return sorted(parts, key=lambda d: d["PartNumber"])  # type: ignore
