@@ -1014,7 +1014,7 @@ def test_log_feedback_invalid_feedback_id(feedback_ids, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize("feedbacks", [10, None, 10.5, np.array([1, 2])])
+@pytest.mark.parametrize("feedbacks", [10, None, 10.5, (1, 2)])
 def test_log_feedback_invalid_feedback(feedbacks, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_feedback(
@@ -1059,12 +1059,17 @@ def test_log_feedback_invalid_timestamps(timestamps, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-# TODO -> add np.ndarrays as possible feedbacks values
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize("random_error", [True, False])
 @pytest.mark.parametrize(
@@ -1072,6 +1077,7 @@ def test_log_feedback_invalid_timestamps(timestamps, cli_obj, caplog):
     [
         [{100: "some-value"}, {100: "some-other-value"}],
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
+        np.array([{100: "some-value"}, {100: "some-other-value"}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1140,13 +1146,20 @@ def test_log_feedback(mock_uuid, feedbacks, random_error, timestamps, sort, tags
 
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "feedbacks",
     [
         [{100: "some-value"}, {100: "some-other-value"}],
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
+        np.array([{100: "some-value"}, {100: "some-other-value"}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1172,13 +1185,20 @@ def test_log_feedback_sample_rate_0_5(mock_uuid, feedbacks, timestamps, sort):
 
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "feedbacks",
     [
         [{100: "some-value"}, {100: "some-other-value"}],
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
+        np.array([{100: "some-value"}, {100: "some-other-value"}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1204,13 +1224,20 @@ def test_log_feedback_sample_rate_0_5_none(mock_uuid, feedbacks, timestamps, sor
 
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "feedbacks",
     [
         [{100: "some-value"}, {100: "some-other-value"}],
         pd.DataFrame({100: ["some-value", "some-other-value"]}),
+        np.array([{100: "some-value"}, {100: "some-other-value"}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1264,7 +1291,7 @@ def test_log_predictions_invalid_name(application, cli_obj, caplog):
 
 
 @pytest.mark.parametrize(
-    "inputs", [[10, {}], {"A": 100}, 10, None, 10.5, b"some binary string", np.array([1, 2])]
+    "inputs", [[10, {}], {"A": 100}, 10, None, 10.5, b"some binary string", (1, 2)]
 )
 def test_log_predictions_invalid_inputs(inputs, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
@@ -1281,9 +1308,7 @@ def test_log_predictions_invalid_inputs(inputs, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-@pytest.mark.parametrize(
-    "outputs", [{"A": 100}, 10, None, 10.5, b"some binary string", np.array([1, 2])]
-)
+@pytest.mark.parametrize("outputs", [{"A": 100}, 10, None, 10.5, b"some binary string", (1, 2)])
 def test_log_predictions_invalid_outputs(outputs, cli_obj, caplog):
     caplog.set_level(logging.ERROR, logger="gantry.client")
     cli_obj.log_predictions(
@@ -1380,12 +1405,17 @@ def test_log_predictions_invalid_timestamps(timestamps, cli_obj, caplog):
     assert "TypeError:" in caplog.text
 
 
-# TODO -> add np.ndarrays as possible inputs/outputs values
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize("random_error", [True, False])
 @pytest.mark.parametrize(
@@ -1393,6 +1423,7 @@ def test_log_predictions_invalid_timestamps(timestamps, cli_obj, caplog):
     [
         [{"C": 300}, {"C": 303}],
         pd.DataFrame({"C": [300, 303]}),
+        np.array([{"C": 300}, {"C": 303}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1400,6 +1431,7 @@ def test_log_predictions_invalid_timestamps(timestamps, cli_obj, caplog):
     [
         [{"A": 100, "B": 200}, {"A": 101, "B": 202}],
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
+        np.array([{"A": 100, "B": 200}, {"A": 101, "B": 202}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1480,13 +1512,20 @@ def test_log_predictions(mock_uuid, inputs, outputs, random_error, timestamps, s
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "outputs",
     [
         [{"C": 300}, {"C": 303}],
         pd.DataFrame({"C": [300, 303]}),
+        np.array([{"C": 300}, {"C": 303}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1494,6 +1533,7 @@ def test_log_predictions(mock_uuid, inputs, outputs, random_error, timestamps, s
     [
         [{"A": 100, "B": 200}, {"A": 101, "B": 202}],
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
+        np.array([{"A": 100, "B": 200}, {"A": 101, "B": 202}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1522,13 +1562,20 @@ def test_log_predictions_sample_rate_0_5(mock_uuid, inputs, outputs, timestamps,
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "outputs",
     [
         [{"C": 300}, {"C": 303}],
         pd.DataFrame({"C": [300, 303]}),
+        np.array([{"C": 300}, {"C": 303}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1536,6 +1583,7 @@ def test_log_predictions_sample_rate_0_5(mock_uuid, inputs, outputs, timestamps,
     [
         [{"A": 100, "B": 200}, {"A": 101, "B": 202}],
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
+        np.array([{"A": 100, "B": 200}, {"A": 101, "B": 202}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1564,13 +1612,20 @@ def test_log_predictions_sample_rate_0_5_none(mock_uuid, inputs, outputs, timest
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
-    "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
+    "timestamps",
+    [
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        None,
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize(
     "outputs",
     [
         [{"C": 300}, {"C": 303}],
         pd.DataFrame({"C": [300, 303]}),
+        np.array([{"C": 300}, {"C": 303}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1578,6 +1633,7 @@ def test_log_predictions_sample_rate_0_5_none(mock_uuid, inputs, outputs, timest
     [
         [{"A": 100, "B": 200}, {"A": 101, "B": 202}],
         pd.DataFrame({"A": [100, 101], "B": [200, 202]}),
+        np.array([{"A": 100, "B": 200}, {"A": 101, "B": 202}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -1684,9 +1740,12 @@ def test_batch_id_population_without_batch(initialized_gantry, tmp_path):
 @pytest.mark.parametrize("method", ["log_predictions", "log_records_prediction", "log_records"])
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
 @mock.patch("gantry.logger.client._batch_success_msg")
+@mock.patch("gantry.logger.client._concurrent_upload_multipart_batch")
 def test_log_predictions_as_batch(
-    mock_batch_success, mock_uuid, version, sort, method, init_gantry_with_api
+    mock_upload, mock_batch_success, mock_uuid, version, sort, method, init_gantry_with_api
 ):
+    mock_upload.return_value = [{"ETag": "working_etag", "PartNumber": 1}]
+
     if method == "log_records":
         size = 40
     else:
@@ -1708,11 +1767,6 @@ def test_log_predictions_as_batch(
                 "key": "x1234",
             },
         )
-        resp.add(
-            resp.PUT,
-            "http://test_url",
-            headers={"Content-Type": "application/json", "ETag": "working_etag"},
-        )
         resp.add(
             resp.POST,
             url=f"{HOST}/api/v1/ingest/file",
@@ -1760,14 +1814,24 @@ def test_log_predictions_as_batch(
 @pytest.mark.parametrize("feedback_id", [{"value": "foo"}, "foo"])
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
 @mock.patch("gantry.logger.client._batch_success_msg")
+@mock.patch("gantry.logger.client._concurrent_upload_multipart_batch")
 def test_log_feedback_as_batch(
-    mock_batch_success, mock_uuid, version, sort, method, init_gantry_with_api, feedback_id
+    mock_upload,
+    mock_batch_success,
+    mock_uuid,
+    version,
+    sort,
+    method,
+    init_gantry_with_api,
+    feedback_id,
 ):
     size = 2
     mock_uuid.side_effect = [str(x) for x in range(size + 1)]
     feedbacks = [{"value": "foo"} for _ in range(size)]
     feedback_ids = [feedback_id for _ in range(size)]
 
+    mock_upload.return_value = [{"ETag": "working_etag", "PartNumber": 1}]
+
     with responses.RequestsMock() as resp:
         resp.add(
             resp.GET,
@@ -1779,11 +1843,6 @@ def test_log_feedback_as_batch(
                 "key": "x1234",
             },
         )
-        resp.add(
-            resp.PUT,
-            "http://test_url",
-            headers={"Content-Type": "application/json", "ETag": "working_etag"},
-        )
         resp.add(
             resp.POST,
             f"{HOST}/api/v1/ingest/file",
@@ -1819,7 +1878,10 @@ def test_log_feedback_as_batch(
 
 
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-def test_batch_completion_fail(mock_uuid, init_gantry_with_api):
+@mock.patch("gantry.logger.client._concurrent_upload_multipart_batch")
+def test_batch_completion_fail(mock_upload, mock_uuid, init_gantry_with_api):
+    mock_upload.return_value = [{"ETag": "working_etag", "PartNumber": 1}]
+
     size = 20
     inputs = [{"input1": i, "input2": 2} for i in range(size)]
     outputs = [i % 2 for i in range(size)]
@@ -1837,11 +1899,6 @@ def test_batch_completion_fail(mock_uuid, init_gantry_with_api):
                 "key": "x1234",
             },
         )
-        resp.add(
-            resp.PUT,
-            "http://test_url",
-            headers={"Content-Type": "application/json", "ETag": "working_etag"},
-        )
         resp.add(
             resp.POST,
             f"{HOST}/api/v1/ingest/file",
@@ -1949,7 +2006,13 @@ def test_log_predictions_and_feedbacks_size_mismatch_timestamps(timestamps, cli_
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize(
-    "timestamps", [None, pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME]]
+    "timestamps",
+    [
+        None,
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize("version", ["0", 0, None])
@@ -1959,6 +2022,7 @@ def test_log_predictions_and_feedbacks_size_mismatch_timestamps(timestamps, cli_
     [
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1966,6 +2030,7 @@ def test_log_predictions_and_feedbacks_size_mismatch_timestamps(timestamps, cli_
     [
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -1973,6 +2038,7 @@ def test_log_predictions_and_feedbacks_size_mismatch_timestamps(timestamps, cli_
     [
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
@@ -2074,7 +2140,13 @@ def test_log_predictions_and_feedbacks_df(
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize(
-    "timestamps", [None, pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME]]
+    "timestamps",
+    [
+        None,
+        pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]),
+        [ANOTHER_TIME, SOME_TIME],
+        np.array([ANOTHER_TIME, SOME_TIME]),
+    ],
 )
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize("version", ["0", 0, None])
@@ -2187,6 +2259,7 @@ def test_log_records_preds_and_feedback_series_coerced(
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
         pd.Series(name="A", data=[200, 201]),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2195,6 +2268,7 @@ def test_log_records_preds_and_feedback_series_coerced(
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
         pd.Series(name="B", data=[300, 301]),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2203,6 +2277,7 @@ def test_log_records_preds_and_feedback_series_coerced(
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
         pd.Series(name="A", data=[100, 101]),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2242,6 +2317,7 @@ def test_log_records_preds_and_feedback(
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
         pd.Series(name="A", data=[200, 201]),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2250,6 +2326,7 @@ def test_log_records_preds_and_feedback(
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
         pd.Series(name="B", data=[300, 301]),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2258,6 +2335,7 @@ def test_log_records_preds_and_feedback(
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
         pd.Series(name="A", data=[100, 101]),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2288,6 +2366,7 @@ def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs,
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
         pd.Series(name="A", data=[200, 201]),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2296,6 +2375,7 @@ def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs,
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
         pd.Series(name="B", data=[300, 301]),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2304,6 +2384,7 @@ def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs,
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
         pd.Series(name="A", data=[100, 101]),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2336,6 +2417,7 @@ def test_log_records_preds_and_feedback_0_5_none(
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
         pd.Series(name="A", data=[200, 201]),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2344,6 +2426,7 @@ def test_log_records_preds_and_feedback_0_5_none(
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
         pd.Series(name="B", data=[300, 301]),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2352,6 +2435,7 @@ def test_log_records_preds_and_feedback_0_5_none(
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
         pd.Series(name="A", data=[100, 101]),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2382,6 +2466,7 @@ def test_log_records_preds_and_feedback_0_0(version, test_inputs, test_outputs,
         [{"B": 300}, {"B": 301}],
         pd.DataFrame.from_dict({"B": [300, 301]}),
         pd.Series(name="B", data=[300, 301]),
+        np.array([{"B": 300}, {"B": 301}]),
     ],
 )
 @pytest.mark.parametrize(
@@ -2390,6 +2475,7 @@ def test_log_records_preds_and_feedback_0_0(version, test_inputs, test_outputs,
         [{"A": 100}, {"A": 101}],
         pd.DataFrame.from_dict({"A": [100, 101]}),
         pd.Series(name="A", data=[100, 101]),
+        np.array([{"A": 100}, {"A": 101}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2422,6 +2508,7 @@ def test_log_records_preds_only(
         [{"A": 200}, {"A": 201}],
         pd.DataFrame.from_dict({"A": [200, 201]}),
         pd.Series(name="A", data=[200, 201]),
+        np.array([{"A": 200}, {"A": 201}]),
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
@@ -2450,15 +2537,15 @@ def test_log_records_feedback_only(
 
 @pytest.mark.parametrize(
     "test_feedbacks",
-    [None, pd.DataFrame(), [], pd.Series()],
+    [None, pd.DataFrame(), [], pd.Series(), np.array([])],
 )
 @pytest.mark.parametrize(
     "test_outputs",
-    [None, pd.DataFrame(), [], pd.Series()],
+    [None, pd.DataFrame(), [], pd.Series(), np.array([])],
 )
 @pytest.mark.parametrize(
     "test_inputs",
-    [None, pd.DataFrame(), [], pd.Series()],
+    [None, pd.DataFrame(), [], pd.Series(), np.array([])],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
@@ -2493,7 +2580,7 @@ def test_log_records_no_data(
 
 @pytest.mark.parametrize(
     "test_feedbacks",
-    [None, pd.DataFrame(), [{"A": 3}, {"A": 3}], pd.Series()],
+    [None, pd.DataFrame(), [{"A": 3}, {"A": 3}], pd.Series(), np.array([])],
 )
 @mock.patch("gantry.logger.client.Gantry._log_predictions_and_feedbacks")
 @mock.patch("gantry.logger.client.Gantry.log_predictions")
