@@ -290,18 +290,21 @@ def test_aggregate_query_num_points(
 
 @pytest.mark.parametrize("tags", [{}, {"foo": "bar"}])
 @pytest.mark.parametrize("filters", [[], [{"some": "filter"}]])
-@mock.patch("gantry.query.core.dataframe._raw_query_cached_request")
-def test_aggregate_query_error(
-    mock_query_data, series_obj_factory_with_filters_and_tags, filters, tags
-):
+def test_aggregate_query_error(series_obj_factory_with_filters_and_tags, filters, tags):
     series_obj = series_obj_factory_with_filters_and_tags("int", filters=filters, tags=tags)
-    mock_query_data.return_value = {"response": "bad", "data": {"query": {"some-attribute": 10}}}
 
-    with pytest.raises(
-        GantryException,
-        match=r"Aggregation query \[some-attribute\] failed on API with this error: Request failed",
-    ):
-        _ = series_obj._aggregate_query("some-attribute", a=10)
+    with responses.RequestsMock() as rsps:
+        rsps.add(
+            responses.POST,
+            "{}/api/v1/aggregate/query".format(ORIGIN),
+            json={"response": "bad", "data": {"query": {"some-attribute": 10}}},
+            status=400,
+        )
+        with pytest.raises(
+            GantryException,
+            match=r"Aggregation query \[some-attribute\] failed: .+",
+        ):
+            _ = series_obj._aggregate_query("some-attribute", a=10)
 
 
 @pytest.mark.parametrize("invalid_num_points", [0, -1, -100])
