@@ -68,3 +68,21 @@ def test_request_rate_limit(caplog):
         with pytest.raises(GantryRequestException) as excinfo:
             api_client_obj.request("GET", "/health", raise_for_status=True)
         assert "Logger has hit the rate limit" in str(excinfo.value)
+
+
+@pytest.mark.parametrize("status_code", list(range(400, 600)))
+def test_request_raise_for_status(status_code):
+    api_client_obj = api_client.APIClient(HOST, FAKE_API_KEY)
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.GET,
+            f"{HOST}/some_resource",
+            headers={"Content-Type": "application/json"},
+            status=status_code,
+            json={
+                "response": "ok",
+            },
+        )
+
+        with pytest.raises(GantryRequestException):
+            api_client_obj.request("GET", "/some_resource", raise_for_status=True)
